import { defineComponent, h, isVue2 } from 'vue-demi';
import '../assets/tailwind.css';

const TwButton = defineComponent({
  name: 'TwButton',
  props: {
    bg: {
      type: String,
      default: 'fill',
      validator: (value: string) => value === 'fill' || value === 'clear',
    },
    bordered: {
      type: Boolean,
      default: false,
    },
    disabled: {
      type: Boolean,
      default: false,
    },
    flex: {
      type: Boolean,
      default: false,
    },
    icon: {
      type: String,
      default: '',
    },
    pill: {
      type: Boolean,
      default: false,
    },
  },
  emits: ['click'],
  setup(props, { emit, slots }) {
    const commonClasses = {
      btn: true,
      'btn-flex': props.flex,
      'opacity-50 cursor-not-allowed': props.disabled,
      '!rounded-full': props.pill,
      border: props.bordered,
      'border-dark': props.bordered && props.bg === 'clear',
      'border-primary-dark': props.bordered && props.bg === 'fill',
      'btn-clear': props.bg === 'clear',
      'btn-fill': props.bg === 'fill',
    };

    function renderButton(hFunction: any) {
      return hFunction(
          'button',
          {
            class: commonClasses,
            attrs: isVue2 ? { disabled: props.disabled } : undefined, // Vue 2 compatibility
            disabled: !isVue2 ? props.disabled : undefined, // Vue 3 compatibility
            on: isVue2 ? { click: (event: Event) => emit('click', event) } : undefined, // Vue 2 event handling
            onClick: !isVue2 ? (event: Event) => emit('click', event) : undefined, // Vue 3 event handling
          },
          [
            hFunction('span', { class: 'btn-contents' }, [
              props.icon
                  ? hFunction(
                      'span',
                      {
                        class: {
                          'icon material-icons': true,
                          'ml-[-2px] mr-1.5': slots.default,
                        },
                      },
                      props.icon
                  )
                  : '',
              slots.default ? hFunction('span', {}, slots.default()) : '',
            ]),
          ]
      );
    }

    return () => renderButton(h);
  },
});

export default TwButton;
